from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from django_library.tools import unique_file_name


class Library(models.Model):
    display_fields = ["name"]
    name = models.CharField(max_length=255, null=True, blank=True)
    is_public = models.BooleanField(default=False)
    created_on = models.DateTimeField(auto_now_add=True, verbose_name=_("Created On"))

    def user_can_read(self, user):
        return self.is_public or library.permission_set.filter(
            Q(Q(user=user) & Q(Q(access_level="O") | Q(access_level="R") | Q(access_level="W")))).exists()

    def user_can_write(self, user):
        return self.permission_set.filter(
            Q(Q(user=user) & Q(Q(access_level="O") | Q(access_level="W")))).exists()

    def __str__(self):
        return self.name


class Permission(models.Model):
    PERMISSION_READ = 'R'
    PERMISSION_WRITE = 'W'
    PERMISSION_OWNER = 'O'

    PERMISSION_CHOICES = (
        (PERMISSION_READ, "Read"),
        (PERMISSION_WRITE, "Write",),
        (PERMISSION_OWNER, "Owner",)
    )

    user = models.ForeignKey(User, on_delete=models.CASCADE)
    library = models.ForeignKey(Library, on_delete=models.CASCADE)
    created_on = models.DateTimeField(auto_now_add=True, verbose_name=_("Created On"))
    access_level = models.CharField(max_length=1, choices=PERMISSION_CHOICES)

    def __str__(self):
        return self.library.name + ' - ' + self.user.username


class File(models.Model):
    uploaded_file = models.FileField(upload_to=unique_file_name)
    original_name = models.CharField(max_length=255, null=True, blank=True)
    name = models.CharField(max_length=255, null=True, blank=True)
    description = models.CharField(max_length=255, null=True, blank=True)
    created_by = models.ForeignKey(User, on_delete=models.CASCADE)
    created_on = models.DateTimeField(auto_now_add=True, verbose_name=_("Created On"))
    library = models.ForeignKey(Library, on_delete=models.CASCADE)

    class Meta:
        ordering = ['-created_on']

    def __str__(self):
        return self.name

    def url(self):
        return self.uploaded_file.url.split('?')[0]



class AppSettings(object):
    def __init__(self, prefix):
        self.prefix = prefix

    def _setting(self, name, dflt):
        from django.conf import settings
        getter = getattr(settings,
                         'ALLAUTH_SETTING_GETTER',
                         lambda name, dflt: getattr(settings, name, dflt))
        return getter(self.prefix + name, dflt)

    @property
    def ADD_FILE_FORM_CLASS(self):
        """
        Add file form
        """
        return self._setting("ADD_FILE_FORM_CLASS", None)

    @property
    def INLINE_FORM(self):
        """
        Add file form
        """
        return self._setting("INLINE_FORM", False)


# Ugly? Guido recommends this himself ...
# http://mail.python.org/pipermail/python-ideas/2012-May/014969.html
import sys  # noqa

app_settings = AppSettings('DJANGO_LIBRARY_')
app_settings.__name__ = __name__
sys.modules[__name__] = app_settings



import uuid


def unique_file_name(self, file_name):
    """
    Generates Unique name for uploaded filess
    :param self: Filefield object Class
    :param file_name: uploaded file name
    :return: file path with the new filename
    """
    # extract filename and extension
    result = file_name.split('.')
    basename, ext = result[0], '.'.join(result[-1:])
    basename = 'uploads/' + basename
    path, file_actual_name = basename.rsplit('/', 1)
    # Generate new name
    new_name = str(uuid.uuid4())
    file_name = new_name + '.' + ext
    self.original_name = file_actual_name + '.' + ext
    return path + "/" + file_name



from django.conf.urls import url

from django_library import views

urlpatterns = [
    url(r'^file/downaload-file/(\d+)/(.+)', views.download_file, name="django_library_download_file"),
    url(r'^file/add/(?P<library_id>\d+)/', views.add_file, name="django_library_add_file"),
    url(r'^file/delete/(?P<file_id>\d+)/', views.delete_file, name="django_library_delete_file"),
]



from importlib import import_module

from django.core import exceptions
from django.http import HttpResponse, HttpResponseForbidden, HttpResponseRedirect, JsonResponse
from django.shortcuts import render, get_object_or_404
from django.utils.encoding import smart_str

from django_library import app_settings
from django_library.forms import FileForm
from django_library.models import File, Library
from django_library.templatetags.django_library import render_library_list


def download_file(request, pk=None, name=None):
    """
    Download file from library after checking if user have permission to download it
    :param request: 
    :param pk: File primary key
    :param name: File name to be downloaded
    :return: file object if is have permission to read the file or FORBIDDEN if not
    """
    file_obj = File.objects.get(pk=pk)
    try:
        library = file_obj.library
    except:
        return HttpResponse({"message": "Something went wrong! please try again later"}, status=500)

    if library.user_can_read(request.user):
        response = HttpResponse(file_obj.uploaded_file.file, content_type="charset=utf-8")
        response.charset = 'utf-8'
        # response['Content-Length'] = os.path.getsize(settings.BASE_DIR + '/' + file_obj.url())
        response['Content-Disposition'] = 'attachment; filename=%s' % smart_str(file_obj.original_name)
        response['Content-Encoding'] = 'utf-8'
        # os.remove(settings.BASE_DIR + '/tmp/' + file_obj.original_name)
        return response
    else:
        return HttpResponseForbidden("You don't have permission to upload to this directory")


def add_file(request, library_id):
    """
    upload and file to library
    :param request: 
    :param library_id: library id to save the file to
    :return: 
    """
    library = get_object_or_404(Library, pk=library_id)
    next_page = request.GET.get("next_url", "/")
    if library.user_can_write(request.user):
        form = get_add_form_class()(request.POST or None, request.FILES or None)
        if form.is_valid():
            form.instance.created_by = request.user
            form.instance.library = library
            form.save()
            if request.is_ajax():
                library.refresh_from_db()
                files_list_html = render_library_list(
                    {"request": request, "next_page": next_page},
                    library)

                result = {"file_list_html": files_list_html}
                return JsonResponse(result)
            else:
                return HttpResponseRedirect(next_page)
        return render(request, "django_library/add_file.html", {"form": form, "library": library})
    else:
        return HttpResponseForbidden()


def delete_file(request, file_id):
    """
    Delete file from library
    :param request: 
    :param file_id: file id to be deleted
    :return: 
    """
    file = get_object_or_404(File, pk=file_id)
    next_page = request.GET.get("next_url", "/")
    if file.library.user_can_write(request.user):
        file.delete()
        return HttpResponseRedirect(next_page)
    else:
        return HttpResponseForbidden()


def get_add_form_class():
    """
    get add file form from settings files
    :return: add file form class
    """
    if not app_settings.ADD_FILE_FORM_CLASS:
        form_cls = FileForm
    else:
        try:
            fc_module, fc_classname = app_settings.ADD_FILE_FORM_CLASS.rsplit('.', 1)
        except ValueError:
            raise exceptions.ImproperlyConfigured('%s does not point to a form'
                                                  ' class'
                                                  % app_settings.ADD_FILE_FORM_CLASS)
        try:
            mod = import_module(fc_module)
        except ImportError as e:
            raise exceptions.ImproperlyConfigured('Error importing form class %s:'
                                                  ' "%s"' % (fc_module, e))
        try:
            fc_class = getattr(mod, fc_classname)
        except AttributeError:
            raise exceptions.ImproperlyConfigured('Module "%s" does not define a'
                                                  ' "%s" class' % (fc_module,
                                                                   fc_classname))
        form_cls = fc_class
    return form_cls



from django import template
from django.template import loader

from django_library import app_settings
from django_library.forms import FileForm

register = template.Library()


@register.simple_tag(takes_context=True)
def render_library(context, library):
    """
    template tag to render library component
    :param context: request context
    :param library: Library object to be rendered
    :return: Html code for library component
    """
    request = context['request']
    csrf_token = context['csrf_token']
    user = request.user
    can_view = library.user_can_read(user)
    can_add_edit = library.user_can_write(user)
    inline_form = app_settings.INLINE_FORM
    # load html template
    template_path = "django_library/full_library.html"
    t = loader.get_template(template_path)
    # prepare context
    context = {
        "library": library,
        "can_view": can_view,
        "can_add_edit": can_add_edit,
        "csrf_token": csrf_token,
        "inline_form": inline_form,
        "request": request,
        "next_page": context['next_page'] if 'next_page' in context else request.path or "/",
    }
    # load file upload foorm if inline form setting is True
    if inline_form:
        form = FileForm()
        context['form'] = form
    html = t.render(context)
    return html


@register.simple_tag(takes_context=True)
def render_library_list(context, library):
    """
    list view for the library
    :param context: request context
    :param library: Library object to be rendered
    :return: Html code for library files list
    """
    request = context['request']
    user = request.user
    can_view = library.user_can_read(user)
    can_add_edit = library.user_can_write(user)
    template_path = "django_library/library_listing.html"
    t = loader.get_template(template_path)
    context = {
        "library": library,
        "request": request,
        "can_view": can_view,
        "next_page": context['next_page'] if 'next_page' in context else request.path or "/",
        "can_add_edit": can_add_edit,
    }
    html = t.render(context)
    return html



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))
setup(
    name='django-files-library',
    description='django files Library component',
    version="0.01.03",
    long_description=open('README.rst').read(),
    author='Ahmad Bazadough',
    author_email='ahmadiga1985@hotmail.com',
    url='https://github.com/ahmadiga/django-files-library',
    download_url='https://github.com/ahmadiga/django-files-library/archive/0.01.tar.gz',
    package_dir={'django_library': 'django_library'},
    packages=find_packages(exclude='test_app'),
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
    ],
)



# Generated by Django 2.0.1 on 2018-02-09 12:21

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_library.tools


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uploaded_file', models.FileField(upload_to=django_library.tools.unique_file_name)),
                ('original_name', models.CharField(blank=True, max_length=255, null=True)),
                ('name', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.CharField(blank=True, max_length=255, null=True)),
                ('created_on', models.DateTimeField(auto_now_add=True, verbose_name='Created On')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Library',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255, null=True)),
                ('is_public', models.BooleanField(default=False)),
                ('created_on', models.DateTimeField(auto_now_add=True, verbose_name='Created On')),
            ],
        ),
        migrations.CreateModel(
            name='Permission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_on', models.DateTimeField(auto_now_add=True, verbose_name='Created On')),
                ('access_level', models.CharField(choices=[('R', 'Read'), ('W', 'Write'), ('O', 'Owner')], max_length=1)),
                ('library', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_library.Library')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='file',
            name='library',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_library.Library'),
        ),
    ]
